; ********** start of bpass.pro
;
; see http://glinda.lrsm.upenn.edu/~weeks/idl
;  for further documentation, though the version there is somewhat different
;
;+
; NAME:
;		bpass
; PURPOSE:
;		Implements a real-space bandpass filter which suppress 
;		pixel noise and long-wavelength image variations while 
;		retaining information of a characteristic size.
;
; CATEGORY:
;		Image Processing
; CALLING SEQUENCE:
;		res = bpass( image, lnoise, lobject )
; INPUTS:
;		image:	The two-dimensional array to be filtered.
;		lnoise: Characteristic lengthscale of noise in pixels.
;			Additive noise averaged over this length should
;			vanish. May assume any positive floating value.
;		lobject: A length in pixels somewhat larger than a typical
;			object. Must be an *_ODD_* valued integer.
;			(Gabe notes: values larger than 19 send the 
;			tails of gaussian mask to such small values that
;			they cannot be represented by the "float" type.
;			The resulting arithmetic underflow is not important,
;			so I have not circumvented it by using the "double" type.)
; OUTPUTS:
;		res:	filtered image.
; PROCEDURE:
;		simple 'wavelet' convolution yields spatial bandpass filtering.
; NOTES:
; MODIFICATION HISTORY: based on dgfilter()
;		Written by David G. Grier, The University of Chicago, 2/93.
;		Greatly revised version DGG 5/95.
;		Added /field keyword JCC 12/95.
;               Memory optimizations and fixed normalization, DGG 8/99.
;
;	This code 'bpass.pro' is copyright 1997, John C. Crocker and 
;	David G. Grier.  It should be considered 'freeware'- and may be
;	distributed freely in its original form when properly attributed.
;-
function bpass, image, lnoise, lobject, field=field, noclip=noclip

;on_error, 2				; go to caller on error

b = float( lnoise )
w = round( lobject > (2. * b) )
N = 2*w + 1

; Gaussian convolution kernel: removes short-wavelength noise:
;Gabe changed this to double precision, Dec 2002, to avoid underflows:
r = double((findgen(N) - w)/(2. * b))
gx = exp( -r^2 ) / (2. * b * sqrt(!pi))
gy = transpose(gx)

; Boxcar average kernel: calculates background
bx = fltarr(N) + 1./N
by = transpose(bx)

if keyword_set( field ) then begin
    if N mod 4 eq 1 then indx = 2*indgen(w+1)$
    		    else indx = 1 + 2*indgen(w)
    nn = n_elements(indx)
    gy = 2. * gy(indx) ; fixes normalization
    by = fltarr(nn) + 1./float(nn)
endif

;Gabe changed this from:
; res = float(image)
; to:
res = double(image)
; do x and y convolutions
g = convol( res, gx )
g = convol( temporary(g), gy )
res = convol( temporary(res), bx )
res = convol( temporary(res), by )
res = g - res

if keyword_set( noclip ) then $
	return, res $
else $
	return, res > 0

end

;*********** end of bpass.pro










