;+
;
; see http://glinda.lrsm.upenn.edu/~weeks/idl
;   for more information
;
; NAME:
;		Feature	
; PURPOSE:
;		Finds and measures roughly CIRCULAR 'features' within 
;		an image.
; CATEGORY:
;		Image Processing
; CALLING SEQUENCE:
;		f = feature( image, diameter [, separation, masscut = masscut,
;			min = min, iterate = iterate, /field, /quiet ] )
; INPUTS:
;		image:	(nx,ny) array which presumably contains some
;			features worth finding
;		diameter: a parameter which should be a little greater than
;			the diameter of the largest features in the image.
;			Diameter MUST BE ODD valued.
;		sep: an optional parameter which specifies the 
;			minimum allowable separation between feature 
;			centers. The default value is diameter+1.
;		min: 	Minimum pixel intensity considered to be a
;                       candidate local maximum (default: 64th percentile).
;		masscut: Discard candidate features with integrated
;                       brightness below this value.
;               pickn:  When this keyword is set to a positive, non-zero number, 
;			the programs will rank candidate features according to 
;			integrated brightness,
;			and retains only this many (the number set by pickn).
;		field: 	Set this keyword if image is actually just one field
;			of an interlaced (e.g. video) image. All the masks
;			will then be constructed with a 2:1 aspect ratio.
;		quiet:	Supress printing of informational messages.
;		iterate: if the refined centroid position is too far from
;			the initial estimate, iteratively recalc. the centroid
;			using the last cetroid to position the mask.  This 
;			can be useful for really noisy data, or data with
;			flat (e.g. saturated) peaks.  USE WITH CAUTION- it
;			may 'climb' hills and give you multiple hits.
;               maxit:  Maximum number of iterations (default = 10).
; OUTPUTS:
;		f(0,*):	this contains the x centroid positions, in pixels.
;		f(1,*): this contains the y centroid positions, in pixels. 
;		f(2,*): this contains the integrated brightness of the 
;			features.
;		f(3,*): this contains the square of the radius of gyration
;			of the features.
;		f(4,*): this contains the eccentricity, which should be 
;			zero for circularly symmetric features and order
;			one for very elongated images.
; SIDE EFFECTS:
;		Displays the number of features found on the screen.
; RESTRICTIONS:
;		To work properly, the image must consist of bright, 
;		circularly symmetric regions on a roughly zero-valued 
;		background. To find dark features, the image should be 
;		inverted and the background subtracted. If the image
;		contains a large amount of high spatial frequency noise,
;		performance will be improved by first filtering the image.
;		BPASS will remove high spatial frequency noise, and 
;		subtract the image background and thus provides a good 
;		complement to using this program. Individual features 
;		should NOT overlap.
; PROCEDURE:
;		First, identify the positions of all the local maxima in
;		the image ( defined in a circular neighborhood with diameter
;		equal to 'diameter' ). Around each of these maxima, place a 
;		circular mask, of diameter 'diameter', and calculate the x & y
;		centroids, the total of all the pixel values, and the radius
;		of gyration and the 'eccentricity' of the pixel values within
;		that mask. If the initial local maximum is found to be more
;		than 0.5 pixels from the centroid and iterate is set, the mask 
;		is moved and the data are re-calculated. This is useful for 
;		noisy data. If the restrictions above are adhered to, and the 
;		features are more than about 5 pixels across, the resulting x 
;		and y values will have errors of order 0.1 pixels for 
;		reasonably noise free images.
;
; *********	       READ THE FOLLOWING IMPORTANT CAVEAT!        **********
;		'feature' is capable of finding image features with sub-pixel
;		accuracy, but only if used correctly- that is, if the 
;		background is subtracted off properly and the centroid mask 
;		is larger than the feature, so that clipping does not occur.
;		It is an EXCELLENT idea when working with new data to plot
;		a histogram of the x-positions mod 1, that is, of the
;		fractional part of x in pixels.  If the resulting histogram
;		is flat, then you're ok, if its strongly peaked, then you're
;		doing something wrong- but probably still getting 'nearest
;		pixel' accuracy.
;
;		For a more quantitative treatment of sub-pixel position 
;		resolution see: 
;		J.C. Crocker and D.G. Grier, J. Colloid Interface Sci.
;		*179*, 298 (1996).
;
; MODIFICATION HISTORY:
;		This code is inspired by feature_stats2 written by
;			David G. Grier, U of Chicago 1992.
;		Written by John C. Crocker, U of Chicago, optimizing 
;			runtime and measurement error JCC 10/93.
;		Added field keyword JCC 4/94. 
;		Added eccentricity parameter JCC 5/95.
;		Added quiet keyword JCC 12/95.
;		Added iteration, fixed up the radius/diameter fiasco and
;		debugging to improve non-centroid data JCC 4/96.
;               Memory and run-time optimizations DGG 8/99.
;               Added PICKN, DGG and E.R. Dufresne 8/99.
;		
;	This code 'feature.pro' is copyright 1999, John C. Crocker and 
;	David G. Grier.  It should be considered 'freeware'- and may be
;	distributed freely in its original form when properly attributed.
;-

;;;
;
;	RSQD: produce a parabolic mask
;
function rsqd, w, h

if n_params() eq 1 then h = w
r2 = fltarr(w,h,/nozero)
xc = float(w-1) / 2.
yc = float(h-1) / 2.
x2 = (findgen(w) - xc)^2
y2 = (findgen(h)- yc)^2

for j = 0, h-1 do begin
    r2[*,j] = x2 + y2[j]
endfor

return,r2
end

;;;
;
;	THETARR: produce a 'theta' mask
;
function thetarr, w

theta = fltarr(w,w,/nozero)
xc = float(w-1) / 2.
yc = float(w-1) / 2.

x = (findgen(w) - xc)
x[xc] = 1e-5
y = (findgen(w) - yc)

for j = 0, w-1 do begin
    theta[*,j] = atan( y[j],x )
endfor

return, theta
end

;;;
;
;	FRACSHIFT: barrel "shifts" a floating point arr by a 
;               fractional pixel amount using bilinear interpolation.
;
function fracshift, im, shiftx, shifty

ipx = fix( shiftx )          ; integer part of x shift
ipy = fix( shifty )          ; integer part of y shift
fpx = shiftx - ipx           ; fractional part of x shift
fpy = shifty - ipy           ; fractional part of y shift
if fpx lt 0 then begin
    fpx=fpx+1 & ipx=ipx-1    ; make sure fractional parts are positive
endif		
if fpy lt 0 then begin
    fpy=fpy+1 & ipy=ipy-1
endif

image = im                   ; preserve input data: use local copy

imagex  = shift( image, ipx+1, ipy   )
imagey  = shift( image, ipx  , ipy+1 )
imagexy = shift( image, ipx+1, ipy+1 )
image   = shift( image, ipx  , ipy   )

res   = ( (1. - fpx) * (1. - fpy) * image   ) + $
	( (     fpx) * (1. - fpy) * imagex  ) + $
	( (1. - fpx) * (     fpy) * imagey  ) + $
	( (     fpx) * (     fpy) * imagexy ) 
	
return,res	
end

;;;
;
;	LMX: identify local maxima in an image.
;       John's version of local_max2, which supports the field keyword
;	and is otherwise identical. 
;
function lmx, image, sep, min=min, field=field

a = byte(image)                 ; dilate requires byte data
range = fix(sep/2)
w = round( 2 * range + 1 )	; width of sample region
s = rsqd( w )			; sample region is circular
good = where( s le range^2 )
mask = bytarr( w, w )
mask[good] = 1b
yrange = range
if keyword_set( field ) then begin
    mask = fieldof( mask, /even )
    yrange = fix(range/2.) + 1
endif	

b = dilate( a, mask, /gray )	; find local maxima in given range
				; Don't include dim pixels
if not keyword_set( min ) then begin
    h = histogram( a )
    goal = 0.64 * total( h )
    min = 0
    val = h[min]
    while val le goal do begin
        min = min + 1
        val = val + h[min]
    endwhile
endif

r = where( a eq b and a ge min, ngood )

if ngood lt 1 then begin
    message, "No local maxima were brighter than MIN", /inf
    return, [-1]
    endif
				; Discard maxima within range of the edge
sz = size( a )
nx = sz[1] & ny = sz[2]
x = r mod nx & y = r / nx
x0 = x - range & x1 = x + range 
y0 = y - yrange & y1 = y + yrange
good = where( x0 ge 0 and x1 lt nx and y0 ge 0 and y1 lt ny, ngood )
if ngood lt 1 then begin
    message, "All local maxima were within range of edge", /inf
    return,[-1]
    endif
r = r[good]
x = x[good] & y = y[good]
x0 = x0[good] & x1 = x1[good]
y0 = y0[good] & y1 = y1[good]
				; Some features may be found two or more
				; times either because of noise or because
				; they have flat peaks. 
				; Find and clear such spurious points.
c = 0b * a
c[r] = a[r]
center = w * yrange + range
for i = 0L, ngood - 1L do begin
    b = c[x0[i]:x1[i],y0[i]:y1[i]]
    b =  temporary(b) * mask	; look only in circular region
    m = max( b, location )
    if location ne center then $
      c[x[i],y[i]] = 0b
endfor
				; Ideally, the above routine would shrink
				; clusters of points down to their center.
				; As written, this will leave the lower
				; right (?) pixel of a cluster.

r = where( c ne 0, ngood )	; What's left are valid maxima.
if ngood lt 1 then message, "All features got set to zero!", /inf

return, r			; return their locations

end

;;;
;
;	FEATURE: John's version of DGG's feature_stats2.
;	which: a) avoids some unnecessary computation (convolutions)
;	b) uses fractional shift techniques to reduce pixel bias in m and Rg
;	c) has the field keyword
;
function feature, a, extent, sep,  $
                  min=min, masscut=masscut, pickn=pickn, $
                  field=field, quiet=quiet, $
                  iterate=iterate, maxit=maxit

extent = fix(extent)
if (extent mod 2) eq 0 then begin
    message,'Requires an odd extent.  Adding 1...',/inf
    extent = extent + 1
endif

if n_params() eq 2 then sep = extent + 1

sz = size( a )
nx = sz[1]
ny = sz[2]
;	Find local maxima
if keyword_set( field ) then $
  if not keyword_set( min ) then loc = lmx( a, sep, /field ) else $ 
  loc = lmx( a, sep, min=min, /field ) $
else $
  if not keyword_set( min ) then loc = lmx( a, sep ) else $
  loc = lmx( a, sep, min=min )

if loc[0] eq -1 then begin
    message, "No local maxima found",/inf
    return, -1
    endif

nmax = n_elements(loc)
x  = float( loc mod nx )
y  = float( loc / nx ) 

xl = x - fix(extent/2) 
xh = xl + extent - 1

;	Set up some masks
rsq = rsqd( extent )
mask = rsq le (float(extent)/2.)^2
xmask = make_array( extent , extent , /float, /index ) mod extent
xmask = xmask - fix(extent/2)  ; center mask at center!
ymask = transpose( xmask )
xmask = xmask * mask
ymask = ymask * mask
rmask = rsq * mask + 1./6.

t = thetarr( extent )
cmask = cos(2.*t) * mask
smask = sin(2.*t) * mask
cen = float(extent-1)/2.
cmask[cen,cen] = 0.
smask[cen,cen] = 0.

;	Extract fields of the masks, if necessary
if keyword_set( field ) then begin
    mask = fieldof(mask,/odd)
    xmask = fieldof(xmask,/odd)
    ymask = fieldof(ymask,/odd)
    rmask = fieldof(rmask,/odd)
    cmask = fieldof(cmask,/odd)
    smask = fieldof(smask,/odd)
    yl = y - fix(extent/4) 
    yh = yl + fix(extent/2) - 1
    yscale = 2
    ycen = cen/2
endif else begin
    yl = y - fix(extent/2) 
    yh = yl + extent - 1
    yscale = 1
    ycen = cen
endelse
	
;	Estimate the mass	
m  = fltarr(nmax, /nozero)
for i = 0, nmax-1 do $
  m[i] = total( a[xl[i]:xh[i],yl[i]:yh[i]] * mask )

if keyword_set( masscut ) then begin
    w = where( m gt masscut, nmax )
    if nmax eq 0 then begin
        message,"MASSCUT: No features left after masscut!",/inf
        return,-1
    endif
    xl = xl[w] & yl = yl[w]
    xh = xh[w] & yh = yh[w]
    x  = x[w]  & y  = y[w]
    m  = m[w]
endif

if keyword_set( pickn ) then begin
    order = sort( m )
    if nmax lt pickn then $
      message, "PICKN: Ignored: Fewer than "+strtrim(pickn,2)+ $
      " features to choose from.",/inf $
    else begin
        good = order[nmax-pickn:*]    
        x = x[good] & y = y[good]
        xl = xl[good] & yl = yl[good]
        xh = xh[good] & yh = yh[good]
        m = m[good]
        nmax = pickn
    endelse
endif

if not keyword_set(quiet) then $
  message, strcompress( nmax ) + ' features found.',/inf

;	Set up some result arrays
xc = fltarr(nmax, /nozero)
yc = fltarr(nmax, /nozero)
rg = fltarr(nmax, /nozero)
e  = fltarr(nmax, /nozero)

;	Calculate feature centers
for i = 0, nmax-1 do begin
    suba = a[xl[i]:xh[i],yl[i]:yh[i]]
    xc[i] = total( suba * xmask )
    yc[i] = total( suba * ymask )
endfor

xc = xc / m
yc = yc / m / yscale

;	Iterate any bad initial estimate.
if keyword_set( iterate ) then begin
    counter = 0
    if not keyword_set( maxit ) then maxit = 10
    repeat begin
        counter = counter + 1	
        w = where( abs(xc) gt 0.6, nbadx )
        if nbadx gt 0 then begin
            dx = round( xc[w] )
            xl[w] = xl[w] + dx
            xh[w] = xh[w] + dx
            x[w] = x[w] + dx
        endif
        w = where( abs(yc) gt 0.6, nbady )
        if nbady gt 0 then begin
            dy = round( yc[w] )
            yl[w] = yl[w] + dy
            yh[w] = yh[w] + dy
            y[w] = y[w] + dy
        endif
        
        ; recalculate centroids for displaced features
        w = where( (abs(xc) gt 0.6) or (abs(yc) gt 0.6), nbad )
        if nbad gt 0 then begin     
	    if not keyword_set( quiet ) then $
		message, "Iterating "+strtrim(nbad,2)+" points",/inf
            for i = 0, nbad-1 do begin
                ndx = w[i]
                suba = a[xl[ndx]:xh[ndx],yl[ndx]:yh[ndx]]
                m[ndx] = total( suba * mask )
                xc[ndx] = total( suba * xmask ) / m[ndx]
                yc[ndx] = (total( suba * ymask ) / m[ndx]) / yscale
            endfor
        endif
    endrep until (nbad eq 0) or (counter eq maxit)
endif

x = x + xc
y = (y + yc) * yscale

;	Calculate features' characteristics
for i = 0, nmax-1 do begin
  suba = fracshift( a[xl[i]:xh[i],yl[i]:yh[i]], -xc[i] , -yc[i] )
  m[i] = total( suba * mask )
  rg[i] = total( suba * rmask ) / m[i]
  den = (m[i] - suba[cen,ycen]) > 1e-6
  e[i] = sqrt(total(suba * cmask)^2 + total(suba * smask)^2) / den
endfor

return, [transpose(x),transpose(y),transpose(m),transpose(rg),transpose(e)]
end

